@@ -24,22 +24,17 @@ import datetime as dt
 
 import pygtk
 pygtk.require("2.0")
-import gtk
+import gtk, pango
 
 import gnomeapplet
 import gobject
 import dbus, dbus.service, dbus.mainloop.glib
 import locale
 
-from configuration import conf, runtime, dialogs
+from configuration import conf, runtime, dialogs, load_ui_file
 
-import stuff
-
-# controllers for other windows
-import widgets
-import idle
-
-import pango
+import widgets, idle
+from utils import stuff, trophies
 
 try:
     import wnck
@@ -194,7 +189,7 @@ class HamsterApplet(object):
                                           ])
 
         # load window of activity switcher and todays view
-        self._gui = stuff.load_ui_file("applet.ui")
+        self._gui = load_ui_file("applet.ui")
         self.window = self._gui.get_object('hamster-window')
         # on close don't destroy the popup, just hide it instead
         self.window.connect("delete_event", lambda *args: self.__show_toggle(False))
@@ -245,12 +240,9 @@ class HamsterApplet(object):
 
         # Load today's data, activities and set label
         self.last_activity = None
-        self.load_day()
-        self.update_label()
+        self.todays_facts = None
 
 
-        # refresh hamster every 60 seconds to update duration
-        gobject.timeout_add_seconds(60, self.refresh_hamster)
         runtime.storage.connect('activities-changed', self.after_activity_update)
         runtime.storage.connect('facts-changed', self.after_fact_update)
         runtime.storage.connect('toggle-called', self.on_toggle_called)
@@ -266,9 +258,15 @@ class HamsterApplet(object):
             self.notification.set_urgency(pynotify.URGENCY_LOW) # lower than grass
 
         self._gui.connect_signals(self)
-
         self.prev_size = None
 
+        self.load_day()
+        gobject.timeout_add_seconds(60, self.refresh_hamster) # refresh hamster every 60 seconds to update duration
+        self.refresh_hamster()
+
+
+
+
     def init_workspace_tracking(self):
         if not wnck: # can't track if we don't have the trackable
             return
@@ -287,16 +285,20 @@ class HamsterApplet(object):
 
             self.update_label()
             self.check_user()
+            trophies.check_ongoing(self.todays_facts)
+        except Exception, e:
+            logging.error("Error while refreshing: %s" % e)
         finally:  # we want to go on no matter what, so in case of any error we find out about it sooner
             return True
 
+
     def update_label(self):
-        if self.last_activity and self.last_activity['end_time'] is None:
-            delta = dt.datetime.now() - self.last_activity['start_time']
+        if self.last_activity and self.last_activity.end_time is None:
+            delta = dt.datetime.now() - self.last_activity.start_time
             duration = delta.seconds /  60
-            label = "%s %s" % (self.last_activity['name'],
+            label = "%s %s" % (self.last_activity.activity,
                                stuff.format_duration(duration, False))
-            self.button.set_text(self.last_activity['name'],
+            self.button.set_text(self.last_activity.activity,
                                  stuff.format_duration(duration, False))
         else:
             label = "%s" % _(u"No activity")
@@ -313,11 +315,11 @@ class HamsterApplet(object):
         now = dt.datetime.now()
         message = None
         if self.last_activity:
-            delta = now - self.last_activity['start_time']
+            delta = now - self.last_activity.start_time
             duration = delta.seconds /  60
 
             if duration and duration % self.notify_interval == 0:
-                message = self.last_activity['name']
+                message = self.last_activity.activity
 
         elif self.notify_on_idle:
             #if we have no last activity, let's just calculate duration from 00:00
@@ -334,9 +336,9 @@ class HamsterApplet(object):
         """sets up today's tree and fills it with records
            returns information about last activity"""
 
-        facts = runtime.storage.get_todays_facts()
+        facts = self.todays_facts = runtime.storage.get_todays_facts()
 
-        if facts and facts[-1]["end_time"] == None:
+        if facts and facts[-1].end_time == None:
             self.last_activity = facts[-1]
         else:
             self.last_activity = None
@@ -355,9 +357,9 @@ class HamsterApplet(object):
 
             by_category = {}
             for fact in facts:
-                duration = 24 * 60 * fact["delta"].days + fact["delta"].seconds / 60
-                by_category[fact['category']] = \
-                              by_category.setdefault(fact['category'], 0) + duration
+                duration = 24 * 60 * fact.delta.days + fact.delta.seconds / 60
+                by_category[fact.category] = \
+                              by_category.setdefault(fact.category, 0) + duration
                 self.treeview.add_fact(fact)
 
             self.treeview.attach_model()
@@ -393,20 +395,20 @@ class HamsterApplet(object):
             self.get_widget("switch_activity").show()
             self.get_widget("start_tracking").hide()
 
-            delta = dt.datetime.now() - activity['start_time']
+            delta = dt.datetime.now() - activity.start_time
             duration = delta.seconds /  60
 
-            if activity['category'] != _("Unsorted"):
-                self.get_widget("last_activity_name").set_text("%s - %s" % (activity['name'], activity['category']))
+            if activity.category != _("Unsorted"):
+                self.get_widget("last_activity_name").set_text("%s - %s" % (activity.activity, activity.category))
             else:
-                self.get_widget("last_activity_name").set_text(activity['name'])
+                self.get_widget("last_activity_name").set_text(activity.activity)
 
 
             self.get_widget("last_activity_duration").set_text(stuff.format_duration(duration) or _("Just started"))
-            self.get_widget("last_activity_description").set_text(activity['description'] or "")
+            self.get_widget("last_activity_description").set_text(activity.description or "")
             self.get_widget("activity_info_box").show()
 
-            self.tag_box.draw(activity["tags"])
+            self.tag_box.draw(activity.tags)
         else:
             self.get_widget("switch_activity").hide()
             self.get_widget("start_tracking").show()
@@ -419,7 +421,7 @@ class HamsterApplet(object):
 
     def delete_selected(self):
         fact = self.treeview.get_selected_fact()
-        runtime.storage.remove_fact(fact["id"])
+        runtime.storage.remove_fact(fact.id)
 
     def __show_toggle(self, is_active):
         """main window display and positioning"""
@@ -504,16 +506,17 @@ class HamsterApplet(object):
 
     def _open_edit_activity(self, row, fact):
         """opens activity editor for selected row"""
-        dialogs.edit.show(self.applet, fact_id = fact["id"])
+        dialogs.edit.show(self.applet, fact_id = fact.id)
 
     def on_today_row_activated(self, tree, path, column):
         fact = tree.get_selected_fact()
+        fact = stuff.Fact(fact.activity,
+                    tags = ", ".join(fact.tags),
+                    category = fact.category,
+                    description = fact.description)
 
-        if fact:
-            runtime.storage.add_fact(fact["name"],
-                                     ", ".join(fact["tags"]),
-                                     category_name = fact["category"],
-                                     description = fact["description"])
+        if fact.activity:
+            runtime.storage.add_fact(fact)
             self.__show_toggle(False)
 
 
@@ -564,7 +567,7 @@ class HamsterApplet(object):
         if state == 0:
             self.refresh_hamster()
         elif self.timeout_enabled and self.last_activity and \
-             self.last_activity['end_time'] is None:
+             self.last_activity.end_time is None:
 
             runtime.storage.stop_tracking(self.dbusIdleListener.getIdleFrom())
 
@@ -587,30 +590,29 @@ class HamsterApplet(object):
         # on switch, update our mapping between spaces and activities
         self.workspace_activities[prev] = self.last_activity
 
-
         activity = None
         if "name" in self.workspace_tracking:
             # first try to look up activity by desktop name
             mapping = conf.get("workspace_mapping")
 
-            parsed_activity = None
+            fact = None
             if new < len(mapping):
-                parsed_activity = stuff.parse_activity_input(mapping[new])
+                fact = stuff.Fact(mapping[new])
 
-            if parsed_activity:
-                category_id = None
-                if parsed_activity.category_name:
-                    category_id = runtime.storage.get_category_id(parsed_activity.category_name)
+                if fact.activity:
+                    category_id = None
+                    if fact.category:
+                        category_id = runtime.storage.get_category_id(fact.category)
 
-                activity = runtime.storage.get_activity_by_name(parsed_activity.activity_name,
-                                                                category_id,
-                                                                resurrect = False)
-                if activity:
-                    # we need dict below
-                    activity = dict(name = activity['name'],
-                                    category = activity['category'],
-                                    description = parsed_activity.description,
-                                    tags = parsed_activity.tags)
+                    activity = runtime.storage.get_activity_by_name(fact.activity,
+                                                                    category_id,
+                                                                    ressurect = False)
+                    if activity:
+                        # we need dict below
+                        activity = dict(name = activity.activity,
+                                        category = activity.category,
+                                        description = fact.description,
+                                        tags = fact.tags)
 
 
         if not activity and "memory" in self.workspace_tracking:
@@ -625,20 +627,21 @@ class HamsterApplet(object):
 
         # check if maybe there is no need to switch, as field match:
         if self.last_activity and \
-           self.last_activity['name'].lower() == activity['name'].lower() and \
-           (self.last_activity['category'] or "").lower() == (activity['category'] or "").lower() and \
-           ", ".join(self.last_activity['tags']).lower() == ", ".join(activity['tags']).lower():
+           self.last_activity.activity.lower() == activity.activity.lower() and \
+           (self.last_activity.category or "").lower() == (activity.category or "").lower() and \
+           ", ".join(self.last_activity.tags).lower() == ", ".join(activity.tags).lower():
             return
 
         # ok, switch
-        runtime.storage.add_fact(activity['name'],
-                                 ", ".join(activity['tags']),
-                                 category_name = activity['category'],
-                                 description = activity['description'])
+        fact = stuff.Fact(activity.activity,
+                          tags = ", ".join(activity.tags),
+                          category = activity.category,
+                          description = activity.description);
+        runtime.storage.add_fact(fact)
 
         if self.notification:
             self.notification.update(_("Changed activity"),
-                                     _("Switched to '%s'") % activity['name'],
+                                     _("Switched to '%s'") % activity.activity,
                                      "hamster-applet")
             self.notification.show()
 
@@ -668,13 +671,14 @@ class HamsterApplet(object):
         self.get_widget("switch_activity").set_sensitive(widget.get_text() != "")
 
     def on_switch_activity_clicked(self, widget):
-        activity_name, temporary = self.new_name.get_value()
-        if not activity_name:
-            return False
+        activity, temporary = self.new_name.get_value()
 
-        runtime.storage.add_fact(self.new_name.get_text().decode("utf8", "replace"),
-                                 self.new_tags.get_text().decode("utf8", "replace"),
-                                 temporary = temporary)
+        fact = stuff.Fact(activity,
+                          tags = self.new_tags.get_text().decode("utf8", "replace"))
+        if not fact.activity:
+            return
+
+        runtime.storage.add_fact(fact, temporary)
         self.new_name.set_text("")
         self.new_tags.set_text("")
         self.__show_toggle(False)
@@ -703,4 +707,6 @@ class HamsterApplet(object):
 
     def on_help_clicked(self, *args):
         gtk.show_uri(gtk.gdk.Screen(), "ghelp:hamster-applet", 0L)
+
+        trophies.unlock("basic_instructions")
         return False
