@@ -45,31 +45,23 @@ def get_prev(selection, model):
 class CategoryStore(gtk.ListStore):
     def __init__(self):
         #id, name, color_code, order
-        gtk.ListStore.__init__(self, int, str, int)
+        gtk.ListStore.__init__(self, int, str)
 
     def load(self):
-        """ Loads activity list from database, ordered by
-            activity_order """
-
         category_list = runtime.storage.get_categories()
 
         for category in category_list:
-            self.append([category['id'],
-                         category['name'],
-                         category['category_order']])
+            self.append([category['id'], category['name']])
 
-        self.unsorted_category = self.append([-1, _("Unsorted"), 999]) # all activities without category
+        self.unsorted_category = self.append([-1, _("Unsorted")]) # all activities without category
 
 
 class ActivityStore(gtk.ListStore):
     def __init__(self):
         #id, name, category_id, order
-        gtk.ListStore.__init__(self, int, str, int, int)
+        gtk.ListStore.__init__(self, int, str, int)
 
     def load(self, category_id):
-        """ Loads activity list from database, ordered by
-            activity_order """
-
         self.clear()
 
         if category_id is None:
@@ -80,8 +72,7 @@ class ActivityStore(gtk.ListStore):
         for activity in activity_list:
             self.append([activity['id'],
                          activity['name'],
-                         activity['category_id'],
-                         activity['activity_order']])
+                         activity['category_id']])
 
 
 class WorkspaceStore(gtk.ListStore):
@@ -92,9 +83,9 @@ class WorkspaceStore(gtk.ListStore):
 formats = ["fixed", "symbolic", "minutes"]
 appearances = ["text", "icon", "both"]
 
-from configuration import runtime, conf
+from configuration import runtime, conf, load_ui_file
 import widgets
-import storage, stuff
+from utils import stuff, trophies
 
 class PreferencesEditor:
     TARGETS = [
@@ -105,9 +96,10 @@ class PreferencesEditor:
 
     def __init__(self, parent = None):
         self.parent = parent
-        self._gui = stuff.load_ui_file("preferences.ui")
+        self._gui = load_ui_file("preferences.ui")
         self.window = self.get_widget('preferences_window')
 
+        # Translators: 'None' refers here to the Todo list choice in Hamster preferences (Tracking tab)
         self.activities_sources = [("", _("None")),
                                    ("evo", "Evolution"),
                                    ("gtg", "Getting Things Gnome")]
@@ -172,15 +164,11 @@ class PreferencesEditor:
                                                 self.TARGETS,
                                                 gtk.gdk.ACTION_DEFAULT|
                                                 gtk.gdk.ACTION_MOVE)
-        self.activity_tree.enable_model_drag_dest(self.TARGETS,
-                                                  gtk.gdk.ACTION_MOVE)
 
         self.category_tree.enable_model_drag_dest(self.TARGETS,
                                                   gtk.gdk.ACTION_MOVE)
 
         self.activity_tree.connect("drag_data_get", self.drag_data_get_data)
-        self.activity_tree.connect("drag_data_received",
-                                   self.drag_data_received_data)
 
         self.category_tree.connect("drag_data_received",
                                    self.on_category_drop)
@@ -273,6 +261,7 @@ class PreferencesEditor:
         while index >= len(self.workspace_mapping):
             self.workspace_mapping.append("")
 
+        value = value.decode("utf8", "replace")
         self.workspace_mapping[index] = value
         conf.set("workspace_mapping", self.workspace_mapping)
         self.workspace_store[path][2] = value
@@ -281,7 +270,6 @@ class PreferencesEditor:
         self.get_widget("shutdown_track").set_active(conf.get("stop_on_shutdown"))
         self.get_widget("idle_track").set_active(conf.get("enable_timeout"))
         self.get_widget("notify_interval").set_value(conf.get("notify_interval"))
-        self.get_widget("keybinding").set_text(conf.get("keybinding"))
 
         self.get_widget("notify_on_idle").set_active(conf.get("notify_on_idle"))
         self.get_widget("notify_on_idle").set_sensitive(conf.get("notify_interval") <=120)
@@ -341,23 +329,6 @@ class PreferencesEditor:
                 self.category_tree.set_cursor((i, ))
             i += 1
 
-    def on_activity_list_drag_motion(self, treeview, drag_context, x, y, eventtime):
-        self.prev_selected_activity = None
-        try:
-            target_path, drop_position = treeview.get_dest_row_at_pos(x, y)
-            model, source = treeview.get_selection().get_selected()
-
-        except:
-            return
-
-        drop_yes = ("drop_yes", gtk.TARGET_SAME_APP, 0)
-        drop_no = ("drop_no", gtk.TARGET_SAME_APP, 0)
-
-        if drop_position == gtk.TREE_VIEW_DROP_AFTER or \
-           drop_position == gtk.TREE_VIEW_DROP_BEFORE:
-            treeview.enable_model_drag_dest(self.TARGETS, gtk.gdk.ACTION_MOVE)
-        else:
-            treeview.enable_model_drag_dest([drop_no], gtk.gdk.ACTION_MOVE)
 
 
     def on_category_list_drag_motion(self, treeview, drag_context, x, y, eventtime):
@@ -379,34 +350,6 @@ class PreferencesEditor:
             treeview.enable_model_drag_dest([drop_no], gtk.gdk.ACTION_MOVE)
 
 
-    def drag_data_received_data(self, treeview, context, x, y, selection,
-                                info, etime):
-        model = treeview.get_model()
-        data = selection.data
-        drop_info = treeview.get_dest_row_at_pos(x, y)
-
-        if drop_info:
-            path, position = drop_info
-            iter = model.get_iter(path)
-            if (position == gtk.TREE_VIEW_DROP_BEFORE
-                or position == gtk.TREE_VIEW_DROP_INTO_OR_BEFORE):
-                logging.debug("insert '%s' before '%s'" % (data, model[iter][3]))
-                runtime.storage.move_activity(int(data), model[iter][3], insert_after = False)
-            else:
-                logging.debug("insert '%s' after '%s'" % (data, model[iter][3]))
-                runtime.storage.move_activity(int(data), model[iter][3], insert_after = True)
-        else:
-            logging.debug("append '%s'" % data)
-
-        if context.action == gtk.gdk.ACTION_MOVE:
-            context.finish(True, True, etime)
-
-
-        self.activity_store.load(self._get_selected_category())
-
-        self.select_activity(int(data))
-
-        return
 
     def on_category_drop(self, treeview, context, x, y, selection,
                                 info, etime):
@@ -467,20 +410,28 @@ class PreferencesEditor:
         id = model[path][0]
         category_id = model[path][2]
 
-        #look for dupes
         activities = runtime.storage.get_category_activities(category_id)
+        prev = None
         for activity in activities:
-            if activity['name'].lower() == new_text.lower():
-                if id == -1: # that was a new category
-                    self.activity_store.remove(model.get_iter(path))
-                self.select_activity(activity['id'])
-                return False
-
+            if id == activity['id']:
+                prev = activity['name']
+            else:
+                # avoid two activities in same category with same name
+                if activity['name'].lower() == new_text.lower():
+                    if id == -1: # that was a new activity
+                        self.activity_store.remove(model.get_iter(path))
+                    self.select_activity(activity['id'])
+                    return False
 
         if id == -1: #new activity -> add
             model[path][0] = runtime.storage.add_activity(new_text.decode("utf-8"), category_id)
         else: #existing activity -> update
-            runtime.storage.update_activity(id, new_text.decode("utf-8"), category_id)
+            new = new_text.decode("utf-8")
+            runtime.storage.update_activity(id, new, category_id)
+            # size matters - when editing activity name just changed the case (bar -> Bar)
+            if prev != new and prev.lower() == new.lower():
+                trophies.unlock("size_matters")
+
         model[path][1] = new_text
         return True
 
@@ -498,8 +449,6 @@ class PreferencesEditor:
             self.activity_store.load(model[iter][0])
 
         #start with nothing
-        self.get_widget('activity_up').set_sensitive(False)
-        self.get_widget('activity_down').set_sensitive(False)
         self.get_widget('activity_edit').set_sensitive(False)
         self.get_widget('activity_remove').set_sensitive(False)
 
@@ -521,18 +470,9 @@ class PreferencesEditor:
 
         # treat any selected case
         unsorted_selected = self._get_selected_category() == -1
-        self.get_widget('activity_up').set_sensitive(False)
-        self.get_widget('activity_down').set_sensitive(False)
-
         self.get_widget('activity_edit').set_sensitive(iter != None)
         self.get_widget('activity_remove').set_sensitive(iter != None)
 
-        if iter != None and not unsorted_selected:
-            first_item = model.get_path(iter) == (0,)
-            self.get_widget('activity_up').set_sensitive(not first_item)
-
-            last_item = model.iter_next(iter) is None
-            self.get_widget('activity_down').set_sensitive(not last_item)
 
     def _del_selected_row(self, tree):
         selection = tree.get_selection()
@@ -692,7 +632,7 @@ class PreferencesEditor:
         """ appends row, jumps to it and allows user to input name """
 
         new_category = self.category_store.insert_before(self.category_store.unsorted_category,
-                                                         [-2, _(u"New category"), -1])
+                                                         [-2, _(u"New category")])
 
         self.categoryCell.set_property("editable", True)
         self.category_tree.set_cursor_on_cell((len(self.category_tree.get_model()) - 2, ),
@@ -705,7 +645,7 @@ class PreferencesEditor:
         """ appends row, jumps to it and allows user to input name """
         category_id = self._get_selected_category()
 
-        new_activity = self.activity_store.append([-1, _(u"New activity"), category_id, -1])
+        new_activity = self.activity_store.append([-1, _(u"New activity"), category_id])
 
         (model, iter) = self.selection.get_selected()
 
@@ -719,26 +659,6 @@ class PreferencesEditor:
         removable_id = self._del_selected_row(self.activity_tree)
         runtime.storage.remove_activity(removable_id)
 
-    def on_activity_up_clicked(self, button):
-        (model, iter) = self.selection.get_selected()
-
-        #previous item
-        prev_iter = get_prev(self.selection, model)
-        runtime.storage.swap_activities(model[iter][0], model[iter][3],
-                                model[prev_iter][0], model[prev_iter][3])
-        model.move_before(iter, prev_iter)
-
-        self.activity_changed(self.selection, model)
-
-    def on_activity_down_clicked(self, button):
-        (model, iter) = self.selection.get_selected()
-
-        next_iter = model.iter_next(iter)
-        runtime.storage.swap_activities(model[iter][0], model[iter][3],
-                                model[next_iter][0], model[next_iter][3])
-        self.activity_store.move_after(iter, next_iter)
-
-        self.activity_changed(self.selection, model)
 
     def on_close_button_clicked(self, button):
         self.close_window()
@@ -788,9 +708,6 @@ class PreferencesEditor:
         conf.set("notify_interval", value)
         self.get_widget("notify_on_idle").set_sensitive(value <= 120)
 
-    def on_keybinding_changed(self, textbox):
-        conf.set("keybinding", textbox.get_text().decode('utf8', 'replace'))
-
     def on_day_start_changed(self, widget):
         day_start = self.day_start.get_time()
         if day_start is None:
